<?php

namespace Drupal\Tests\onlyone\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\Core\Language\LanguageInterface;
use Drupal\onlyone\OnlyOne;

/**
 * Tests the Language class methods.
 *
 * @group onlyone
 * @coversDefaultClass \Drupal\onlyone\OnlyOne
 */
class OnlyOneTest extends UnitTestCase {

  /**
   * A entity type manager instance.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $entityTypeManager;

  /**
   * A connection instance.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $connection;

  /**
   * A language manager instance.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $languageManager;

  /**
   * A config factory instance.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $configFactory;

  /**
   * A entity query instance.
   *
   * @var \Drupal\Core\Entity\Query\QueryFactory|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $entityQuery;

  /**
   * The OnlyOne Object.
   *
   * @var Drupal\onlyone\OnlyOne
   */
  protected $onlyOne;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    // Entity type manager mock.
    $this->entityTypeManager = $this->createMock('Drupal\Core\Entity\EntityTypeManagerInterface');
    // Connection mock.
    $this->connection = $this->createMock('Drupal\Core\Database\Connection');
    // Language manager mock.
    $this->languageManager = $this->createMock('Drupal\Core\Language\LanguageManagerInterface');
    // Config factory mock.
    $this->configFactory = $this->createMock('Drupal\Core\Config\ConfigFactoryInterface');
    // Entity query mock.
    $this->entityQuery = $this->createMock('Drupal\Core\Entity\Query\QueryFactory');

    // Creating the object.
    $this->onlyOne = new OnlyOne($this->entityTypeManager, $this->connection, $this->languageManager, $this->configFactory, $this->entityQuery);

  }

  /**
   * Tests the content types label list with OnlyOne::getContentTypesList().
   *
   * @param string $expected
   *   The expected result from calling the function.
   * @param array $content_types_list
   *   A list with content types objects.
   *
   * @covers ::getContentTypesList
   * @dataProvider providerGetContentTypesList
   */
  public function testGetContentTypesList($expected, array $content_types_list) {
    // EntityStorage mock.
    $entity_storage = $this->createMock('Drupal\Core\Entity\EntityStorageInterface');
    // loadMultiple mock.
    $entity_storage->expects($this->any())
      ->method('loadMultiple')
      ->willReturn($content_types_list);
    // Mocking getStorage method.
    $this->entityTypeManager->expects($this->any())
      ->method('getStorage')
      ->with('node_type')
      ->willReturn($entity_storage);

    // Testing the function.
    $this->assertEquals($expected, $this->onlyOne->getContentTypesList());
  }

  /**
   * Data provider for testGetContentTypesList().
   *
   * @return array
   *   An array of arrays, each containing:
   *   - 'expected' - Expected return from getContentTypesList().
   *   - 'content_types_list' - A list with content types objects.
   *
   * @see getContentTypesList()
   */
  public function providerGetContentTypesList() {

    $expected = [
      [
        'article' => 'Article',
        'page' => 'Basic Page',
        'test' => 'Test Content Type',
      ],
      [
        'article' => 'Article',
        'blog' => 'Blog Post',
        'house' => 'House',
        'page' => 'Basic Page',
      ],
    ];
    // Getting the number of tests.
    $number_of_tests = count($expected);
    $content_types_list = [];
    // Creating the array of objects.
    for ($i = 0; $i < $number_of_tests; $i++) {
      $j = 0;

      // Creating the array of objects.
      foreach ($expected[$i] as $id => $label) {
        // EntityInterface mock.
        $content_types_list[$i][$j] = $this->createMock('Drupal\Core\Entity\EntityInterface');
        // Mocking id method.
        $content_types_list[$i][$j]->expects($this->any())
          ->method('id')
          ->willReturn($id);
        // Mocking label method.
        $content_types_list[$i][$j]->expects($this->any())
          ->method('label')
          ->willReturn($label);

        $j++;
      }
    }

    $tests['content types list 1'] = [$expected[0], $content_types_list[0]];
    $tests['content types list 2'] = [$expected[1], $content_types_list[1]];

    return $tests;
  }

  /**
   * Tests the OnlyOne::getTemporaryContentTypesTableName() method.
   *
   * @param string $expected
   *   The expected result from calling the function.
   *
   * @covers ::getTemporaryContentTypesTableName
   * @dataProvider providerGetTemporaryContentTypesTableName
   */
  public function testGetTemporaryContentTypesTableName($expected) {
    // Mocking isMultilingual.
    $this->connection->expects($this->any())
      ->method('queryTemporary')
      ->willReturn($expected);

    // Testing the function.
    $this->assertEquals($expected, $this->onlyOne->getTemporaryContentTypesTableName());
  }

  /**
   * Data provider for testGetTemporaryContentTypesTableName().
   *
   * @return array
   *   An array of arrays, each containing:
   *   - 'expected' - Expected return from getTemporaryContentTypesTableName().
   *
   * @see testExistsNodesContentType()
   */
  public function providerGetTemporaryContentTypesTableName() {
    $tests[] = ['tablename1'];
    $tests[] = ['tablename2'];

    return $tests;
  }

  /**
   * Tests the existence of nodes with OnlyOne::existsNodesContentType().
   *
   * @param string $expected
   *   The expected result from calling the function.
   * @param string $content_type
   *   The content type to check.
   * @param bool $multilingual
   *   Content types configured to have onlyone node.
   * @param string $current_language
   *   The current language selected from the interface.
   * @param array $nids
   *   An array of ids. The keys and values are entity ids.
   *
   * @covers ::existsNodesContentType
   * @dataProvider providerExistsNodesContentType
   */
  public function testExistsNodesContentType($expected, $content_type, $multilingual, $current_language, array $nids) {
    // QueryInterface mock.
    $query = $this->createMock('\Drupal\Core\Entity\Query\QueryInterface');
    // QueryInterface::condition mock.
    $query->method('condition')
      ->withConsecutive(['type', $content_type], ['langcode', $current_language])
      ->willReturnOnConsecutiveCalls($this->returnSelf(), $this->returnSelf());

    // Mocking isMultilingual.
    $this->languageManager->expects($this->any())
      ->method('isMultilingual')
      ->willReturn($multilingual);

    // LanguageInterface mock.
    $language = $this->createMock('Drupal\Core\Language\LanguageInterface');
    // Mocking getName method.
    $language->expects($this->any())
      ->method('getId')
      ->willReturn($current_language);

    // Mocking getLanguage method.
    $this->languageManager->expects($this->any())
      ->method('getCurrentLanguage')
      ->willReturn($language);

    // Mocking get method.
    $this->entityQuery->expects($this->any())
      ->method('get')
      ->with('node')
      ->willReturn($query);

    // Mocking execute method.
    $query->expects($this->any())
      ->method('execute')
      ->willReturn($nids);

    // Testing the function.
    $this->assertEquals($expected, $this->onlyOne->existsNodesContentType($content_type));
  }

  /**
   * Data provider for testExistsNodesContentType().
   *
   * @return array
   *   An array of arrays, each containing:
   *   - 'expected' - Expected return from existsNodesContentType().
   *   - 'content_type' - The content type to check.
   *   - 'multilingual' - Whether the site is multilingual or not.
   *   - 'current_language' - The current language selected from the interface.
   *   - 'nids' - An array of ids. The keys and values are entity ids.
   *
   * @see testExistsNodesContentType()
   */
  public function providerExistsNodesContentType() {
    $tests['node multilingual'] = [1, 'page', TRUE, 'en', [1 => 1]];
    $tests['node multilingual'] = [1, 'page', TRUE, 'es', [1 => 1]];
    $tests['node multilingual'] = [2, 'article', TRUE, 'en', [2 => 2]];
    $tests['node multilingual'] = [2, 'article', TRUE, 'es', [2 => 2]];
    $tests['node multilingual'] = [3, 'blog', TRUE, 'en', [3 => 3]];
    $tests['node multilingual'] = [4, 'book', TRUE, 'es', [4 => 4]];

    $tests['node not multilingual'] = [1, 'page', FALSE, 'en', [1 => 1]];
    $tests['node not multilingual'] = [1, 'page', FALSE, 'es', [1 => 1]];
    $tests['node not multilingual'] = [2, 'article', FALSE, 'en', [2 => 2]];
    $tests['node not multilingual'] = [2, 'article', FALSE, 'es', [2 => 2]];
    $tests['node not multilingual'] = [3, 'blog', FALSE, 'en', [3 => 3]];
    $tests['node not multilingual'] = [4, 'book', FALSE, 'es', [4 => 4]];

    $tests['not node multilingual'] = [0, 'page', TRUE, 'en', []];
    $tests['not node multilingual'] = [0, 'page', TRUE, 'es', []];
    $tests['not node multilingual'] = [0, 'article', TRUE, 'en', []];
    $tests['not node multilingual'] = [0, 'article', TRUE, 'es', []];
    $tests['not node multilingual'] = [0, 'blog', TRUE, 'en', []];
    $tests['not node multilingual'] = [0, 'book', TRUE, 'es', []];

    $tests['not node not multilingual'] = [0, 'page', FALSE, 'en', []];
    $tests['not node not multilingual'] = [0, 'page', FALSE, 'es', []];
    $tests['not node not multilingual'] = [0, 'article', FALSE, 'en', []];
    $tests['not node not multilingual'] = [0, 'article', FALSE, 'es', []];
    $tests['not node not multilingual'] = [0, 'blog', FALSE, 'en', []];
    $tests['not node not multilingual'] = [0, 'book', FALSE, 'es', []];

    return $tests;
  }

  /**
   * Tests content type config deletion with OnlyOne::deleteContentTypeConfig().
   *
   * @param string $expected
   *   The expected result from calling the function.
   * @param string $content_type
   *   Content type machine name to delete through
   *   OnlyOne::eleteContentTypeConfig().
   * @param string $content_types
   *   Content types configured to have onlyone node.
   *
   * @covers ::deleteContentTypeConfig
   * @dataProvider providerDeleteContentTypeConfig
   */
  public function testDeleteContentTypeConfig($expected, $content_type, $content_types) {
    // ImmutableConfig mock.
    $config = $this->createMock('Drupal\Core\Config\ImmutableConfig');
    // ImmutableConfig::get mock.
    $config->expects($this->any())
      ->method('get')
      ->with('onlyone_node_types')
      ->willReturn($content_types);

    // ImmutableConfig::set mock.
    $config->expects($this->any())
      ->method('set')
      ->with('onlyone_node_types', $this->anything())
      ->willReturnSelf();

    // ImmutableConfig::save mock.
    $config->expects($this->any())
      ->method('save')
      ->willReturnSelf();

    // Mocking getEditable method.
    $this->configFactory->expects($this->any())
      ->method('getEditable')
      ->with('onlyone.settings')
      ->willReturn($config);

    // Testing the function.
    $this->assertEquals($expected, $this->onlyOne->deleteContentTypeConfig($content_type));
  }

  /**
   * Data provider for testDeleteContentTypeConfig().
   *
   * @return array
   *   An array of arrays, each containing:
   *   - 'expected' - Expected return from deleteContentTypeConfig().
   *   - 'content_types' - Content types configured to have onlyone node.
   *   - 'content_type' - The content type to delete from the config.
   *
   * @see testDeleteContentTypeConfig()
   */
  public function providerDeleteContentTypeConfig() {
    // Content types configured to have onlyone node.
    $content_types[1] = [
      'page',
      'blog',
      'article',
    ];

    $content_types[2] = [
      'house',
      'car',
    ];

    $content_types[3] = [
      'person',
      'dog',
      'cat',
      'bird',
    ];

    $tests['existing content type'] = [TRUE, 'page', $content_types[1]];
    $tests['existing content type'] = [TRUE, 'car', $content_types[2]];
    $tests['existing content type'] = [TRUE, 'dog', $content_types[3]];
    $tests['non existing content type'] = [FALSE, 'book', $content_types[1]];
    $tests['non existing content type'] = [FALSE, 'plane', $content_types[2]];
    $tests['non existing content type'] = [FALSE, 'horse', $content_types[3]];

    return $tests;
  }

  /**
   * Tests the language label with OnlyOne::getLanguageLabel().
   *
   * @param string $expected
   *   The expected result from calling the function.
   * @param string $language_code
   *   The language code to run through OnlyOne::getLanguageLabel().
   *
   * @covers ::getLanguageLabel
   * @dataProvider providerGetLanguageLabel
   */
  public function testGetLanguageLabel($expected, $language_code) {
    // LanguageInterface mock.
    $language = $this->createMock('Drupal\Core\Language\LanguageInterface');
    // Mocking getName method.
    $language->expects($this->any())
      ->method('getName')
      ->willReturn($expected);

    // Mocking getLanguage method.
    $this->languageManager->expects($this->any())
      ->method('getLanguage')
      ->with($language_code)
      ->willReturn($language);

    // Testing the function.
    $this->assertEquals($expected, $this->onlyOne->getLanguageLabel($language_code));
  }

  /**
   * Data provider for testGetLanguageLabel().
   *
   * @return array
   *   An array of arrays, each containing:
   *   - 'expected' - Expected return from getLanguageLabel().
   *   - 'language_code' - The language code.
   *
   * @see testGetLanguageLabel()
   */
  public function providerGetLanguageLabel() {
    $tests['und langcode'] = ['Not specified', LanguageInterface::LANGCODE_NOT_SPECIFIED];
    $tests['zxx langcode'] = ['Not applicable', LanguageInterface::LANGCODE_NOT_APPLICABLE];
    $tests['es langcode'] = ['Es', 'es'];
    $tests['en langcode'] = ['En', 'en'];

    return $tests;
  }

  /**
   * Tests language label for empty language with OnlyOne::getLanguageLabel().
   *
   * The empty string as language_code is a special situation, you can read more
   * at: https://stackoverflow.com/q/49466415/3653989 .
   *
   * @covers ::getLanguageLabel
   */
  public function testGetLanguageLabelForEmptyLanguage() {
    // LanguageInterface mock.
    $language = $this->createMock('Drupal\Core\Language\LanguageInterface');
    // Mocking getName method.
    $language->expects($this->any())
      ->method('getName')
      ->willReturn('Not specified');

    // Mocking getLanguage method.
    $this->languageManager->expects($this->any())
      ->method('getLanguage')
      ->with(LanguageInterface::LANGCODE_NOT_SPECIFIED)
      ->willReturn($language);

    // Testing the function.
    $this->assertEquals('Not specified', $this->onlyOne->getLanguageLabel(''));
  }

}
